__author__ = 'jiyue'
import matplotlib.pyplot as plt
import numpy as np




def plot_confusion_matrix(cm, classes, normalize=False, title='Confusion Matrix', cmap=plt.cm.Blues):
    import itertools

    plt.imshow(cm, interpolation='nearest', cmap=cmap)
    plt.title(title)
    plt.colorbar()

    tick_marks = np.arange(len(classes))
    plt.xticks(tick_marks, classes, rotation=45)
    plt.yticks(tick_marks, classes)

    if normalize:
        cm = cm.astype('float') / cm.sum(axis=1)[:, np.newaxis]
        print("Normalized confusion matrix")
    else:
        print('Confusion matrix, without normalization')

    print(cm)
    thresh = cm.max() / 2.
    for i, j in itertools.product(range(cm.shape[0]), range(cm.shape[1])):
        plt.text(j, i, cm[i, j],
                 horizontalalignment="center",
                 color="white" if cm[i, j] > thresh else "black")
    plt.tight_layout()
    plt.ylabel('True label')
    plt.xlabel('Predicted label')


def compute_corr(df, corr_value):
    cor = df.corr()
    cor.loc[:, :] = np.tril(cor, k=-1)  # below main lower triangle of an array
    cor = cor.stack()
    return cor[(cor > corr_value) | (cor < -corr_value)]


def nan_value_analysis(df):
    return df.select_dtypes(include=['float']).describe().T.assign(
        missing_pct=df.apply(lambda x: (len(x) - x.count()) / float(len(x))))



__author__ = 'jiyue'

from abc import ABCMeta, abstractmethod


class MainBootstrap(object):
    __metaclass__ = ABCMeta

    def __init__(self):
        pass

    @abstractmethod
    def load_data_and_split(self,
                            split_mode='ratio',
                            in_format='dataframe',
                            in_postfix='csv'
                            ):
        pass

    @abstractmethod
    def go_binning(self, feature_list, target, event_identify, binning_mode, binning_spec_with, binned_other_value,
                   binned_num=5):
        pass

    @abstractmethod
    def train_score_card(self, feature_list, target, event_identify, feature_method='discrete', model='lr'):
        pass

    @abstractmethod
    def predict_score_card(self, feature_list, target):
        pass

    @abstractmethod
    def model_evaluation(self, score_col_name, target, event_identify, ks_bin_num=100):
        pass

    def go_bootstrap(self,
                     split_mode,
                     in_format,
                     in_postfix,
                     feature_list,
                     target,
                     event_identify,
                     binning_mode,
                     binning_spec_with,
                     binned_other_value,
                     binned_num,
                     feature_method,
                     model,
                     score_col_name,
                     ks_bin_num):
        self.load_data_and_split(split_mode, in_format, in_postfix)



__author__ = 'jiyue'
import pandas as pd
from sklearn.datasets import dump_svmlight_file, load_svmlight_file
from sklearn.externals.joblib import Memory

mem = Memory("~/.svmmem")


def compute_missing_pct(dataframe, dtype):
    dataframe.select_dtypes(include=[dtype]).describe().T \
        .assign(missing_pct=dataframe.apply(lambda x: (len(x) - x.count()) / float(len(x))))


def dump_to_svm_format_file(data_frame, label='label', file_name='svm-output.libsvm'):
    dummy = pd.get_dummies(data_frame)
    y = data_frame[label]
    mat = dummy.as_matrix()
    dump_svmlight_file(mat, y, file_name)


def load_svm_format_file(path):
    data = load_svmlight_file(path)
    return data[0], data[1]



__author__ = 'jiyue'


def compute_ks(data):
    sorted_list = data.sort_values(['predict'], ascending=[True])

    total_bad = sorted_list['label'].sum(axis=None, skipna=None, level=None, numeric_only=None) / 3
    total_good = sorted_list.shape[0] - total_bad

    # print "total_bad = ", total_bad
    # print "total_good = ", total_good

    max_ks = 0.0
    good_count = 0.0
    bad_count = 0.0
    for index, row in sorted_list.iterrows():
        if row['label'] == 3:
            bad_count += 1.0
        else:
            good_count += 1.0

        val = bad_count / total_bad - good_count / total_good
        max_ks = max(max_ks, val)

    return max_ks



# coding: utf-8

# In[1]:


import numpy as np
import pandas as pd
import scipy
from scipy import sparse
from sklearn.pipeline import make_pipeline
from sklearn.grid_search import GridSearchCV
import matplotlib.pyplot as plt
import scikit_credit

plt.style.use('ggplot')

# transformers for category variables
from sklearn.preprocessing import LabelBinarizer
from sklearn.preprocessing import MultiLabelBinarizer
from sklearn.preprocessing import LabelEncoder
from sklearn.preprocessing import OneHotEncoder


# transformers for numerical variables
from sklearn.preprocessing import MinMaxScaler
from sklearn.preprocessing import StandardScaler
from sklearn.preprocessing import Normalizer


# transformers for combined variables
from sklearn.decomposition import PCA
from sklearn.preprocessing import PolynomialFeatures


# user-defined transformers
from sklearn.preprocessing import FunctionTransformer


# classification models
from sklearn.ensemble import RandomForestClassifier
from sklearn.linear_model import LogisticRegression


# evaluation
from sklearn.metrics import scorer

from sklearn.linear_model import LogisticRegression


# In[2]:

df = pd.read_csv(u'/home/jiyue/qddata/dm_tmp_user_shouxinzengxin_d4.csv', sep='\t')


# In[4]:

df.drop(['order_time', 'gender', 'add_info_time'], axis=1, inplace=True)


# In[5]:

df.drop_duplicates(inplace=True)

from scikit_credit import plot

df.drop(
    ['score', 'operatorvoices_total_time_avg_everyday_6m', 'ss_bank_avg_income_6m', 'ss_operatorbasic_extendjointdt',
     'operatorbills_billmonthamt_avg_6m'], axis=1, inplace=True)


# In[10]:

df.fillna(0, inplace=True)




# In[14]:

from scikit_credit import encoder



# # 首先进行适当的降采样

# In[15]:

good_bad_odd = 10

total_nums = df.shape[0]
bad_nums = df[df['label'] == 1].shape[0]
good_nums = df[df['label'] == 0].shape[0]

origin_g_b_rate = (1.0 * good_nums / bad_nums)  # 原始好坏样本比例
df_good_users = df[df['label'] == 0].sample(bad_nums * good_bad_odd)
df_bad_users = df[df['label'] == 1]

print df_good_users.shape[0]
print df_bad_users.shape[0]


# In[16]:

frames = [df_good_users, df_bad_users]
df = pd.concat(frames)

y_src = df['label']
X_src = df.drop(['label'], axis=1)

woe_encoder = encoder.WoeEncoder(bin_width=8)
woe_encoder.fit_transform(X_src.values, y_src.values)


# In[33]:

binned_X = woe_encoder._X_binned
print binned_X.max()
print binned_X.min()

# In[31]:

df_binned_X = pd.DataFrame(data=binned_X, columns=X_src.columns.values)
df_binned_X['id_age'].value_counts()


# In[20]:

# df_binned_X['ss_bank_avg_income_6m'] =  df_binned_X['ss_bank_avg_income_6m'].astype('category')
# df_binned_X['ss_operatorbasic_extendjointdt'] =  df_binned_X['ss_operatorbasic_extendjointdt'].astype('category')
# df_binned_X['operatorbills_billmonthamt_avg_6m'] =  df_binned_X['operatorbills_billmonthamt_avg_6m'].astype('category')
# df_binned_X['operatorvoices_total_time_avg_everyday_6m'] =  df_binned_X['operatorvoices_total_time_avg_everyday_6m'].astype('category')
df_binned_X['id_age'] = df_binned_X['id_age'].astype('category')
# df_binned_X['score'] =  df_binned_X['score'].astype('category')



# In[21]:

df_binned_dummy_x = pd.get_dummies(df_binned_X)

from sklearn.metrics import confusion_matrix
from sklearn.metrics import recall_score, precision_score, accuracy_score, roc_auc_score
from sklearn.ensemble import RandomForestClassifier
from sklearn.model_selection import GridSearchCV, cross_val_score, train_test_split, cross_val_predict, StratifiedKFold, \
    KFold
from sklearn.dummy import DummyClassifier
from sklearn.metrics import roc_curve, auc, roc_auc_score, accuracy_score

dummy_classifier = DummyClassifier(random_state=0, strategy='uniform')
from sklearn.metrics import recall_score, precision_score, accuracy_score, roc_auc_score

lr = LogisticRegression()
total = 0
fpr_sum, tpr_sum = 0, 0

X_train, X_test, y_train, y_test = train_test_split(df_binned_dummy_x.values, y_src)
print y_src.value_counts()

lr.fit(X_train, y_train)
rf_predict = lr.predict(X_test)
rf_predict_proba = lr.predict_proba(X_test)[:, 1]

print 'rf:recall:{},precision:{},accuracy:{},roc_auc:{}'.format(recall_score(y_test, rf_predict),
                                                                precision_score(y_test, rf_predict),
                                                                accuracy_score(y_test, rf_predict),
                                                                roc_auc_score(y_test, rf_predict_proba)
                                                                )

fpr, tpr, thresholds = roc_curve(y_test, rf_predict_proba)
coef = lr.coef_[0]
columns = df_binned_dummy_x.columns.values

plt.figure()
cf = confusion_matrix(y_test, rf_predict)
from scikit_credit import plot

plot.plot_confusion_matrix(cf, ['0', '1'])
plt.show()


def get_column_name(column):
    column_arr = column.split('_')
    num = str(column_arr[len(column_arr) - 1])
    return column[:-len(num) - 1]


print len(columns), columns
print woe_encoder._binned_range
with open('/home/jiyue/Desktop/output_encoding', 'w') as f:
    index = 0
    # fea_idx是特征索引,features_list是每个特征的分箱
    for fea_idx, features_list in enumerate(woe_encoder._binned_range):
        for binned_index, feature_item_range in enumerate(features_list):
            col_name = get_column_name(columns[index])
            f.write(col_name + '$' + str(int(feature_item_range[0]))
                    + '~' + str(int(feature_item_range[1]))
                    + '  ' + str(coef[index])
                    + ' ' + str(woe_encoder._woe[binned_index][fea_idx])
                    + '\n')

            print binned_index, feature_item_range, index, columns[index]
            index += 1
    print index



# In[ ]:




# In[ ]:




# In[ ]:




# In[ ]:



__author__ = 'jiyue'
import numpy as np

# 相关性分析
def compute_corr(df, corr_value):
    cor = df.corr()
    cor.loc[:, :] = np.tril(cor, k=-1)  # below main lower triangle of an array
    cor = cor.stack()
    print cor[(cor > corr_value) | (cor < -corr_value)]



# -*- coding:utf-8 -*-
import math

__author__ = 'jiyue'
import requests
import json
import pandas as pd
import numpy as np
from scikit_credit import encoder

'''
zm_score_list = [584, 652, 671, 690, 715, 854]
age_list = [18, 24, 27, 30, 33, 51]
income_amount_list = [3275, 5000, 7200, 12375, 240000]
billmonthamt_list = [941, 1463, 1996, 2729, 35053]

data = dict()
data['version'] = 'v1'
data['data'] = list()
data['data'].append(dict())
# data['data'][0]['zm_score'] = 710
# data['data'][0]['age'] = 35

for zm_score in zm_score_list:
    data['data'][0]['zm_score'] = zm_score
    for age in age_list:
        data['data'][0]['age'] = age
        for income_amount in income_amount_list:
            data['data'][0]['income_amount'] = income_amount
            for billmonthamt in billmonthamt_list:
                data['data'][0]['billmonthamt'] = billmonthamt
                res = requests.post('http://localhost:8098/fc/lqaddinfo', data=json.dumps(data))
                if res.status_code == 200:
                    res_json = json.loads(res.text)
                    print "zm_score:" + str(zm_score) + " " + "age:" + str(age) + " " + "income_amount:" + str(
                        income_amount) + " " + "billmonthamt:" + str(billmonthamt) + "     " + "score:" + str(
                        res_json['data'][0]['score'])
'''

df = pd.read_csv(u'/home/jiyue/qddata/银行收入和运营商.csv', sep=',')
df_good_users = df[df['label'] == 0].sample(6000)
df_bad_users = df[df['label'] == 1]

frames = [df_good_users, df_bad_users]
df = pd.concat(frames)

y_src = df['label']
X_src = df.drop(['label', 'gender', 'ss_bank_expense_income_diff_all'], axis=1)

# 去掉一个最大值,一个最小值
woe_encoder = encoder.WoeEncoder(binning_mode='ef', bin_width=5)
woe_encoder.fit_transform(X_src.values, y_src.values)
print woe_encoder._features_iv
for item in woe_encoder._binned_range:
    print item
    print '\n'


def cal_qd_score_credit_pass(a_score, zm_score, b_score, td_score, ivs_score):
    p = 15
    q = 15
    d = 25
    c = 20
    b = 50

    theta = 10
    alpha = 5

    if math.isnan(zm_score) or zm_score == 0:
        return 0.0

    if math.isnan(a_score) or a_score == 0:
        a_score = 639.0

    if math.isnan(b_score) or b_score == 0:
        b_score = 596.0

    if math.isnan(td_score):
        td_score = 30.0

    if math.isnan(ivs_score):
        ivs_score = 70.0

    if td_score > 100:
        td_score = 100.0

    if ivs_score == 0:
        ivs_score = 70.0

    e = p * (td_score / 100) + q * (100 - ivs_score) / 100
    N_A = 1.0 * (a_score - 350) / (928 - 350)
    N_B = 1.0 * (b_score - (-59)) / (858 - (-59))
    N_Z = 1.0 * (zm_score - 350) / (928 - 350)
    N_R = 0.7
    return str(N_A) + ',' + str(N_B) + ',' + str(N_Z) + ',' + str(N_R) + ',' + str(e) + ',' + str((int)(
        theta * (alpha * N_R + b * N_A + c * N_Z + d * N_B - e)))


def cal_qd_score_credit_reject_600plus(a_score, zm_score, b_score, td_score, ivs_score):
    p = 20
    q = 20
    d = 35
    c = 20
    b = 40

    theta = 8
    alpha = 5

    if math.isnan(zm_score) or zm_score == 0:
        return 0
    if math.isnan(a_score) or a_score == 0:
        a_score = 639.0
    if math.isnan(b_score) or b_score == 0:
        b_score = 596.0

    if math.isnan(td_score):
        td_score = 30.0
    if math.isnan(ivs_score):
        ivs_score = 70.0

    if td_score > 100:
        td_score = 100.0
    if ivs_score == 0:
        ivs_score = 70.0

    e = p * (td_score / 100) + q * (100 - ivs_score) / 100
    N_A = 1.0 * (a_score - 350) / (928 - 350)
    N_B = 1.0 * (b_score - (-59)) / (858 - (-59))
    N_Z = 1.0 * (zm_score - 350) / (928 - 350)
    N_R = 0.5
    return str(N_A) + ',' + str(N_B) + ',' + str(N_Z) + ',' + str(N_R) + ',' + str(e) + ',' + str((int)(
        theta * (alpha * N_R + b * N_A + c * N_Z + d * N_B - e)))


df = pd.read_csv('/home/jiyue/qddata/dm_tmp_qd_src_score_credit_chayihua.csv', sep='\t')
print 'a_score,zm_score,b_score,tdscore,ivs_score,N_A,N_B,N_Z,N_R,e,qd_score'
for index, row in df.iterrows():
    if math.isnan(row['tdscore']) or math.isnan(row['zm_antifraud_score']):
        continue

    print str(row['a_score']) + ',' + str(row['zm_score']) + ',' + str(row['b_score']) + ',' + str(
        row['tdscore']) + ',' + str(row['zm_antifraud_score']) + ',' + str(
        cal_qd_score_credit_reject_600plus(row['a_score'], row['zm_score'], row['b_score'], row['tdscore'],
                                           row['zm_antifraud_score']))
